min_plus_convolution_linear

Function min_plus_convolution_linear 

pub fn min_plus_convolution_linear<T>(a: &[T], b: &[T]) -> Vec<T>
where T: Signed + TryFrom<usize>,
Computes convolution when at least one input is finite and linear.

Panics

Panics unless at least one input is finite and linear, or an index cannot be represented by T.