pub fn min_plus_convolution_linear<T>(a: &[T], b: &[T]) -> Vec<T>Expand description
Computes convolution when at least one input is finite and linear.
Panics
Panics unless at least one input is finite and linear, or an index cannot
be represented by T.